% Plotting the hemodynamic response function (hrf) using fmridesign

hrf_parameters=[5.4 5.2 10.8 7.35 0.35]
time=(0:240)/10;
hrf0=fmridesign(time,0,[1 0],[],hrf_parameters);
plot(time,squeeze(hrf0(:,1,1)),'LineWidth',2)
xlabel('time (seconds)')
ylabel('hrf')
title('Glover hrf model')

% Making the design matrices using fmridesign

frametimes=(0:119)*3;
slicetimes=[0.14 0.98 0.26 1.10 0.38 1.22 0.50 1.34 0.62 1.46 0.74 1.58 0.86];
eventid=kron(ones(10,1),[1; 2]);
eventimes=(0:19)'*18+9;
duration=ones(20,1)*9;
height=ones(20,1);
events=[eventid eventimes duration height] 

S=kron(ones(10,1),kron([0 0; 1 0; 0 0; 0 1],ones(3,1)));
X_cache=fmridesign(frametimes,slicetimes,events,[],hrf_parameters);
X_cache=fmridesign(frametimes,slicetimes,  []  , S,hrf_parameters);

plot(squeeze(X_cache(:,:,1,5)),'LineWidth',2)
legend('Hot','Warm')
xlabel('frame number')
ylabel('response')
title('Hot and Warm responses')

% Analysing one run with fmrilm

contrast=[1  0;
          0  1;
          1 -1];
exclude=[1 2];
which_stats=[1 1 1 1 1];



input_file='brian_ha_19971124_1_100326_mri_MC.mnc';
output_file_base=['ha_100326_hot';
                  'ha_100326_wrm';
                  'ha_100326_hmw'];
[df1 p]=fmrilm(input_file, output_file_base, X_cache, contrast, exclude, which_stats);

m=fmris_read_image('ha_100326_hmw_mag_t.mnc',4,1);
imagesc(m.data'); colorbar; axis xy;

a=fmris_read_minc(input_file,4,1);
imagesc((m.data.*(a.data>500))'); colorbar; axis xy;

m=fmris_read_image('ha_100326_hmw_mag_t.mnc',4:6,1);
mm=reshape(permute(m.data,[2 1 3 4]),m.dim(1),prod(size(m.data))/m.dim(1));
imagesc(mm.*repmat(a.data'>500,1,3)); colorbar; axis xy;

contrast=[0  0  0 1 0 0;
          0  0  0 0 1 0;
          0  0  0 0 0 1]
which_stats=[0 0 0 1];
output_file_base='ha_100326_hot';
fwhm_rho='ha_100326_hot_rho.mnc';
fmrilm(input_file,output_file_base,X_cache,contrast,exclude,which_stats,fwhm_rho);

% A linear effect of temperature
 
temperature=[45.5 35.0 49.0 38.5 42.0 49.0 35.0 42.0 38.5 45.5 ...
             38.5 49.0 35.0 45.5 42.0 45.5 38.5 42.0 35.0 49.0]';
       
Temperature=35:3.5:49;
subplot(2,2,1)
plot(Temperature,1:5,'LineWidth',2);
xlabel('Temperature'); ylabel('Response');
title('(a) Linear temperature effect');
subplot(2,2,2)
plot(Temperature,10-((1:5)-4).^2,'LineWidth',2);
xlabel('Temperature'); ylabel('Response');
title('(b) Quadratic temperature effect');
subplot(2,2,3)
plot(Temperature,10-((1:5)-4).^2+((1:5)-3).^3,'LineWidth',2);
xlabel('Temperature'); ylabel('Response');
title('(c) Cubic temperature effect');       
subplot(2,2,4)
plot(Temperature,[1 4 3 5 3.5],'LineWidth',2);
xlabel('Temperature'); ylabel('Response');
title(['(d) Quartic = arbitrary temperature effect']);       

events=[zeros(20,1)+1 eventimes duration ones(20,1);
        zeros(20,1)+2 eventimes duration temperature] 

contrast=[0  1;
          1 49;
          1 35;
          0 14];

events=[zeros(20,1)+1 eventimes duration ones(20,1);
        zeros(20,1)+2 eventimes duration temperature; 
        zeros(20,1)+3 eventimes duration temperature^2] 
contrast=[0 0 1];

contrast=[1  0  0  0  0;
          0  1  0  0  0;
          0  0  1  0  0;
          0  0  0  1  0;
          0  0  0  0  1];
contrast=[eye(5) ones(5,4)];
which_stats=[0 0 0 1];

contrast=[.8 -.2 -.2 -.2 -.2;
         -.2  .8 -.2 -.2 -.2;
         -.2 -.2  .8 -.2 -.2;
         -.2 -.2 -.2  .8 -.2;
         -.2 -.2 -.2 -.2  .8];
contrast=[eye(5)-ones(5)/5 ones(5,4)];
which_stats=[0 0 0 1];

contrast=[-7.0 -3.5  0  3.5 7.0];
which_stats=[1 1 1];

% Combining runs/sessions/subjects with multistat

contrast=[1 -1];
which_stats=[1 1 1];



